#include "OneWire.h"
#include "pins_arduino.h"
#include "DS28EC20.h"

/*
* Modified for using with Arduino.
* Supports DS28EC20
* 	Kerry Wong, 2011. 
* http://www.kerrywong.com
* Origial code can be found at:
* http://mbed.org/users/Wimpie/libraries/OneWireEEPROM
*
* Copyright (C) <2011> Wim De Roeve <wim312@gmail.com>
*
* OneWireEEPROM is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* OneWireEEPROM is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* If not, see <http://www.gnu.org/licenses/>.
*/
DS28EC20::DS28EC20()
{
}

DS28EC20::DS28EC20(unsigned int pinNo)
{
	DS28EC20::setPin(pinNo);
}

void DS28EC20::setPin(unsigned int pinNo)
{
	_mem.setPin(pinNo);
}

byte* DS28EC20::searchFirst()
{
	byte* ptr;
	if (!_mem.search(_addr)) {		
		ptr = NULL;
		_mem.reset_search();
	} else {
		ptr = _addr;
	}
		
	return ptr;
}

bool DS28EC20::init()
{
	DS28EC20::searchFirst();
	
	if (OneWire::crc8(_addr, 7) != _addr[7]) return false;
	if (_addr[0] != DS28EC20::DS28EC20_FAMILY_CODE) return false;	

	return true;
}

void DS28EC20::readMem(byte* dest, unsigned int addr, unsigned int size)
{
	byte TA1 = (byte) (addr & 0xFF);
	byte TA2 = (byte) ((addr >> 8) & 0xFF) ;
  
	_mem.reset();
	_mem.select(_addr);
  
	_mem.write(DS28EC20::CMD_READ_MEMORY);
	_mem.write(TA1);
	_mem.write(TA2);

	for (int memPtr = 0; memPtr < size; memPtr++) {
		dest[memPtr] = _mem.read();
	}
}

//Note that addr+size must lie within a single page (256 bits or 32 bytes)
//for data spanning multiple pages, need to use multiple writeMem calls.
bool DS28EC20::writeMem(byte* src, unsigned int addr, unsigned int size)
{
	byte TA1 = (byte) (addr & 0xFF);
	byte TA2 = (byte) ((addr >> 8) & 0xFF);

	_mem.reset();  
	_mem.select(_addr);

	_mem.write(DS28EC20::CMD_WRITE_SCRATCHPAD);
	_mem.write(TA1);
	_mem.write(TA2);
 
	for (int i = 0 ; i < size; i++) {
		_mem.write(src[i]);
	}
  
	_mem.reset();
	_mem.select(_addr);
  
	_mem.write(DS28EC20::CMD_READ_SCRATCHPAD);
  
	byte tempVal = _mem.read();  
	if (tempVal != TA1) return false;

	tempVal = _mem.read();  
	if (tempVal != TA2) return false;
  
	byte ES = _mem.read();
  
	for (int i = 0 ; i < size; i++) {
		tempVal = _mem.read();
		if (tempVal != src[i]) return false;
	}
  
	delay(10);
	_mem.reset();
	_mem.select(_addr);
  
	_mem.write(DS28EC20::CMD_COPY_SCRATCHPAD);
	_mem.write(TA1);
	_mem.write(TA2);
	_mem.write(ES);
	delay(10);
  
	_mem.reset();
  
	return true;
}
