#ifndef DS28EC20_h
#define DS28EC20_h

#include "OneWire.h"
#include <WProgram.h>
#include <inttypes.h>

/*
* Modified for using with Arduino.
* Supports DS28EC20
* 	Kerry Wong, 2011. 
* http://www.kerrywong.com
* Origial code can be found at:
* http://mbed.org/users/Wimpie/libraries/OneWireEEPROM
*
* Copyright (C) <2011> Wim De Roeve <wim312@gmail.com>
*
* OneWireEEPROM is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* OneWireEEPROM is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* If not, see <http://www.gnu.org/licenses/>.
*/

class DS28EC20
{
	private:
		OneWire _mem;
		byte _addr[8];

		byte* searchFirst();
	public:
		static const int DS28EC20_FAMILY_CODE = 0x43;

		static const int CMD_WRITE_SCRATCHPAD = 0x0F;
		static const int CMD_READ_SCRATCHPAD = 0xAA;
		static const int CMD_COPY_SCRATCHPAD = 0x55;
		static const int CMD_READ_MEMORY = 0xF0;
		static const int CMD_EXTENDED_READ_MEMORY = 0xA5;

		static const int REG_MEMORY_BLOCK_LOCK = 0xA1E;
		static const int REG_REGISTER_PAGE_LOCK = 0xA1F;

		static const int WRITE_PROTECTION = 0x55;
		static const int EPROM_MODE = 0xAA;

		static const int PAGE_SIZE = 0x20;
		static const int NUM_OF_PAGES = 0x50;
		static const int MEM_SIZE = PAGE_SIZE * NUM_OF_PAGES;

		DS28EC20();
		DS28EC20(unsigned int pinNo);
		
		void setPin(unsigned int  pinNo);
		bool init();
		void readMem(byte* dest, unsigned int addr, unsigned int size);
		bool writeMem(byte* src, unsigned int addr, unsigned int size);
};
#endif
